# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os


class ShellAutocomplete:
    """Set autocompletion."""

    AUTOCOMPLETION_START_TAG = '### narrenschiff-autocompletion-start ###\n'
    AUTOCOMPLETION_END_TAG = '### narrenschiff-autocompletion-end ###\n'

    def autocompletion_script(self):
        """
        Return autocompletion script.

        :return: Autocompletion script
        :rtype: ``list`` of ``str``
        """
        return ''.join([
            '\n\n',
            ShellAutocomplete.AUTOCOMPLETION_START_TAG,
            'eval "$(_NARRENSCHIFF_COMPLETE=source_bash narrenschiff)"\n',
            ShellAutocomplete.AUTOCOMPLETION_END_TAG
        ])

    def get_config_file(self):
        """
        Get type of the config file.

        :return: Name of the config file
        :rtype: ``str``

        This method returns either ``.bashrc`` (which will later be edited for
        the current user), or ``activate`` which symbolises activate script of
        your virtualenv.
        """
        if os.getenv('VIRTUAL_ENV'):
            return os.getenv('VIRTUAL_ENV'), 'bin/activate'
        return '~', '.bashrc'

    def get_abs_path(self, *args):
        """
        Join arguments, and return absolute path.

        :param args: Paths to be joined
        :type args: ``list`` of ``str``
        :return: Absolute path
        :rtype: ``str``

        If the first argument is tilde (~) then the method will expand user.
        """
        if args[0] == '~':
            return os.path.join(os.path.expanduser('~'), *args[1:])
        return os.path.join(*args)

    def read_file(self, path):
        """
        Read configuration file.

        :param path: Path to the file to read
        :type path: ``str``
        :return: File as a list
        :rtype: ``list`` of ``str``
        """
        with open(path, 'r') as f:
            return f.readlines()

    def autocompletion_enabled(self, config):
        """
        Check if the file contains the autocompletion tag.

        :param config: Configuration file
        :type config: ``list`` of ``str``
        :return: Confirmation whether file contains autocompletion tag
        :rtype: ``bool``
        """
        if ShellAutocomplete.AUTOCOMPLETION_START_TAG in config:
            return True
        return False

    def add_autocompletion(self, path):
        """
        Add autocompletion to the config file.

        :param path: Path to configuration file
        :type path: ``str``
        :return: Void
        :rtype: ``None``
        """
        config = self.read_file(path)
        if not self.autocompletion_enabled(config):
            with open(path, 'a') as f:
                f.write(self.autocompletion_script())

    def add(self):
        """
        Add narrenschiff autocomplete to shell config.

        :return: Void
        :rtype: ``None``
        """
        path, cnftype = self.get_config_file()
        config_path = self.get_abs_path(path, cnftype)
        self.add_autocompletion(config_path)



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import uuid
import shutil
import tempfile

import yaml
from jinja2 import Environment
from jinja2 import FileSystemLoader

from narrenschiff.filters import filters
from narrenschiff.config import Keychain
from narrenschiff.chest import AES256Cipher
from narrenschiff.common import is_yaml
from narrenschiff.common import Singleton
from narrenschiff.log import NarrenschiffLogger


logger = NarrenschiffLogger()


class TemplateException(Exception):
    """Use for exceptions regarding template manipulation."""

    pass


class VarsFileNotFoundError(Exception):
    """Missing files containing variables."""

    pass


class Vars:
    """Manipulate files containing variables."""

    def __init__(self, name, template_directory):
        """
        Initialize :class:`narrenschiff.templating.Vars` objects.

        :param name: ``str`` name used for file search
        """
        self.name = name
        self.template_directory = template_directory

    def _find_var_files(self):
        """
        Find all files containing variables for the templates.

        :return: List of paths to var files.
        :rtype: ``list`` of ``str``
        """
        logger.info('Searching for var files')
        paths = []

        has_dir = False
        has_file = False

        # Find name.yaml or name.yml file
        # yaml has presendance
        for ext in ['yaml', 'yml']:
            file_name = "{}.{}".format(self.name, ext)
            file_path = os.path.join(self.template_directory, file_name)
            logger.debug(f'Searching for {file_name} on {file_path}')
            if os.path.isfile(file_path):
                has_file = True
                break

        # Find name directory
        directory_path = os.path.join(self.template_directory, self.name)
        logger.debug(f'Searching for vars on {directory_path}')
        if os.path.isdir(directory_path):
            paths.extend(self._walk_directory(directory_path))
            has_dir = True

        if has_file:
            paths.append(file_path)

        logger.debug(f'Has var file: {has_file}')
        logger.debug(f'Has var directory: {has_dir}')

        if not has_file and not has_dir:
            logger.error(
                f'No var file found on {self.template_directory}'
            )
            logger.error(f'No var files found on {directory_path}')
            raise VarsFileNotFoundError

        logger.info('Var files found!')
        for path in paths:
            logger.debug(f'Var files found on {path}')
        return paths

    def _walk_directory(self, directory):
        """
        Walk the directory.

        :param directory: Absolute path to the directory
        :type directory: ``str``
        :return: List of files
        :rtype: ``list`` of ``str``
        """
        paths = []
        for root, dirs, files in os.walk(directory):
            for file in files:
                if is_yaml(file):
                    paths.append(os.path.join(root, file))
        return paths

    def _load_vars(self, filepaths):
        """
        Load content of var files into a list.

        :param filepaths: List of file paths of var files
        :type filepaths: ``list`` of ``str``
        :return: List of dictionary values
        :rtype: ``list`` of ``dict``

        **Important:** Empty files return ``None``. This method filters out
        those values.
        """
        vars = []
        for filepath in filepaths:
            with open(filepath, 'r') as f:
                vars.append(yaml.safe_load(f))
        return [var for var in vars if var]

    def load_vars(self):
        """
        Load variables.

        :return: Variables to be used for template rendering
        :rtype: ``dict``
        """
        var_files = self._find_var_files()
        plain_vars = self._load_vars(var_files)

        return plain_vars


class PlainVars(Vars):
    """Load cleartext variable files."""

    NAME = 'vars'

    def __init__(self, template_directory):
        super().__init__(
            name=PlainVars.NAME, template_directory=template_directory
        )


class ChestVars(Vars):
    """Load chest files."""

    NAME = 'chest'

    def __init__(self, template_directory):
        super().__init__(
            name=ChestVars.NAME, template_directory=template_directory
        )

    def load_vars(self):
        var_files = self._find_var_files()
        ciphertext_vars = self._load_vars(var_files)

        keychain = Keychain()
        cipher = AES256Cipher(keychain)
        for var in ciphertext_vars:
            for key, val in var.items():
                var[key] = cipher.decrypt(val)

        return ciphertext_vars


class SecretmapVars(Vars):
    """Load secretmap files."""

    NAME = 'secretmap'

    def __init__(self, template_directory):
        super().__init__(
            name=SecretmapVars.NAME, template_directory=template_directory
        )


class Template(metaclass=Singleton):
    """Load and manipulate templates and template environment."""

    def set_course(self, path):
        """
        Prepare Jinja2 templating environment.

        :param path: Path of the ``tasks.yaml`` file
        :type path: ``str``
        :return: Void
        :rtype: ``None``

        Directory containing the ``course`` (e.g. ``tasks.yaml``) file will
        be the directory from where the templates are taken to be rendered.
        """
        logger.info('Preparing templating environment')
        if not (path.endswith('.yml') or path.endswith('.yaml')):
            exception = 'Files must end with ".yaml" or ".yml" extension'
            raise TemplateException(exception)

        self.tasks_file = os.path.abspath(path)
        self.template_directory = os.path.abspath(os.path.dirname(path))

        logger.debug(f'Task file: {self.tasks_file}')
        logger.debug(f'Template directory {self.template_directory}')

        loader = FileSystemLoader(self.template_directory)
        self.env = Environment(loader=loader)

        # Add filters
        for name, func in filters.items():
            self.env.filters[name] = func

        self.tmp = ''
        self.vars = self.load_vars()

    def render(self, path):
        """
        Render template on the given path.

        :param path: Path to the template file
        :type path: ``str``
        :return: Rendered template
        :rtype: ``str``
        """
        logger.debug(f'Rendering template on {path}')
        template = self.env.get_template(path)
        return template.render(**self.vars)

    def load_vars(self):
        """
        Load variables.

        :return: Variables to be used for template rendering
        :rtype: ``dict``

        This is the order in which files containing variables are loaded:

        1. Load ``vars.yaml`` if it exists
        2. Load all files from the ``vars/`` directory if it exists
        3. Load and decrypt all variables from ``chest.yaml``
        4. Load all files from the ``chest/`` directory if it exists
        5. Load all variables from ``secretmap.yaml``
        6. Merge all files

        **Important:** Files must not contain duplicate variable names!
        """
        logger.info('Load vars into templates from all var types')
        vars = [
            *PlainVars(self.template_directory).load_vars(),
            *ChestVars(self.template_directory).load_vars(),
            *SecretmapVars(self.template_directory).load_vars(),
        ]

        vars_temp = []
        for var in vars:
            vars_temp.extend(list(var.keys()))

        duplicates = self.find_duplicates(vars_temp)
        if duplicates:
            exception = 'Duplicate variable names for these variables: "{}"'
            raise TemplateException(exception.format(', '.join(duplicates)))

        variables = {}
        for dct in vars:
            variables.update(dct)

        logger.debug(f'Vars loaded: {variables}')
        return variables

    def find_duplicates(self, values):
        """
        Find duplicate keys.

        :param values: List of keys from var and chest files.
        :type values: ``list`` of ``str``
        :return: List of duplicates
        :rtype: ``list`` of ``str``
        """
        seen = set()
        duplicates = []
        for val in values:
            if val not in seen:
                seen.add(val)
            else:
                duplicates.append(val)
        return duplicates

    def render_all_files(self):
        """
        Render all templates from the directory.

        :return: Void
        :rtype: ``None``

        Templates are copied to ``/tmp`` and location of rendered templates
        is saved in ``self.tmp``.
        """
        self.tmp = os.path.join(tempfile.gettempdir(), str(uuid.uuid4()))
        os.makedirs(self.tmp)
        files_root = os.path.join(self.template_directory, 'files')
        for root, dirs, files in os.walk(files_root):
            root_path = root.replace(files_root, '')
            root_path = root_path[1:] if root_path else root_path
            os.makedirs(os.path.join(self.tmp, root_path), exist_ok=True)
            for file in files:
                if is_yaml(file):
                    rendered = self.render(os.path.join('files', root_path, file))  # noqa
                    with open(os.path.join(self.tmp, root_path, file), 'w') as f:  # noqa
                        f.write(rendered)

    def clear_templates(self):
        """
        Delete all templates from the ``/tmp`` directory.

        :return: Void
        :rtype: ``None``
        """
        logger.info(f'Deleting rendered templates at {self.tmp}')
        shutil.rmtree(self.tmp)
        logger.info('Done!')



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class KubectlDryRunMixin:
    """Add dry run to kubectl related modules."""

    @property
    def dry_run(self):
        return '--dry-run=server'  # none, server, client, -o yaml

    def dry_run_supported(self, cmd):
        whitelist = [
            'run',
            'apply',
            'delete',
            'create',
            'scale',
            'autoscale',
            'patch',
            'replace',
        ]

        if cmd.split()[1] in whitelist:
            return True
        return False



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import uuid
import yaml
import click


def touch(path):
    """Create a file, but don't overwrite it."""
    with open(path, 'a'):
        pass


@click.command()
@click.option('--location', help='Relative path to course project directory.')
@click.option('--autogenerate',
              required=False,
              is_flag=True,
              help='Autogenerate key and spice, and add to .narrenschiff.yaml')
def dock(location, autogenerate):
    """
    Start a project. Use autogenerate to automatically generate key and spice
    in users home directory. This is compatible only with UNIX systems. Be
    careful with this command, it can overwrite your existing configuration!
    \f

    :param location: Path to the course directory
    :type location: ``str``
    :return: Void
    :rtype: ``None``
    """
    abs_location = os.path.abspath(location)
    os.makedirs(abs_location, mode=0o755, exist_ok=True)

    touch(os.path.join(os.getcwd(), ".narrenschiff.yaml"))

    for conf in ["vars.yaml", "chest.yaml", "secretmap.yaml"]:
        touch(os.path.join(abs_location, conf))

    if autogenerate:
        key = str(uuid.uuid4())
        spice = str(uuid.uuid4())

        root_project_name = os.path.basename(os.getcwd())
        project_config_path = os.path.join(
            os.path.expanduser('~'),
            f'.{root_project_name}'
        )

        key_path = os.path.join(project_config_path, 'key.txt')
        spice_path = os.path.join(project_config_path, 'spice.txt')

        os.makedirs(project_config_path)

        with open(key_path, 'w') as f:
            f.write(key)

        with open(spice_path, 'w') as f:
            f.write(spice)

        dot_narrenschiff = {
            'key': key_path.replace(os.path.expanduser('~'), '~'),
            'spice': spice_path.replace(os.path.expanduser('~'), '~')
        }

        with open('.narrenschiff.yaml', 'w') as f:
            yaml.dump(dot_narrenschiff, f)

        click.echo(f'Credentials are located in {project_config_path}')



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click

import narrenschiff
from narrenschiff.cli.env import env
from narrenschiff.cli.lint import lint
from narrenschiff.cli.dock import dock
from narrenschiff.cli.sail import sail
from narrenschiff.cli.chest import chest
from narrenschiff.cli.secretmap import secretmap
from narrenschiff.cli.autocomplete import autocomplete
from narrenschiff.log import NarrenschiffLogger


@click.group()
@click.option('--verbosity',
              'verbosity',
              required=False,
              type=int,
              help='Set verbosity (levels available are from 1 to 5)')
@click.version_option(narrenschiff.__version__, message='%(version)s')
def narrenschiff(verbosity=0):
    """
    Base command.
    \f

    :return: Void
    :rtype: ``None``
    """
    logger = NarrenschiffLogger()
    logger.set_verbosity(verbosity)


narrenschiff.add_command(env)
narrenschiff.add_command(lint)
narrenschiff.add_command(dock)
narrenschiff.add_command(sail)
narrenschiff.add_command(chest)
narrenschiff.add_command(secretmap)
narrenschiff.add_command(autocomplete)



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import click
import yaml

from narrenschiff.task import Task
from narrenschiff.task import TasksEngine
from narrenschiff.templating import Template
from narrenschiff.config import Keychain
from narrenschiff.config import KubectlContext
from narrenschiff.secretmap import Secretmap


@click.command()
@click.option('--set-course', 'course', help='Path to your YAML course file.')
@click.option(
    '--follow-beacons',
    'beacons',
    required=False,
    type=str,
    help='Execute tasks marked only by given beacons (comma separated list).')
@click.option('--dry-run', required=False, is_flag=True)
def sail(course, beacons, dry_run):
    """
    Turn tasks into actions.
    \f

    :param course: The path to ``course`` file. A file containing tasks
        specified in the YAML format. Tasks are ``executable`` pieces of
        configuration. Course is Jinja2 templated file (as are all the
        kubernetes manifest files)
    :type course: ``str``
    :return: Void
    :rtype: ``None``
    """
    context = KubectlContext()

    template = Template()
    template.set_course(course)

    secretmap = Secretmap(Keychain(), os.path.dirname(course))

    tasks = _import_course(os.path.basename(course), template)

    template.render_all_files()
    secretmap.render_all_files()

    try:
        beacons = set(beacons.split(','))
    except AttributeError:
        beacons = set()

    engine = TasksEngine(tasks, beacons, dry_run)

    try:
        _check_or_switch(context)
        engine.run()
    finally:
        _check_or_switch(context)
        template.clear_templates()
        secretmap.clear_all_files()


def _check_or_switch(context):
    """
    Change context for the kubectl command.

    :param context: Context configuration class
    :type context: :class:`narrenschiff.config.KubectlContext`
    :return: Void
    :rtype: ``None``
    """
    if context.use:
        context.switch()


def _import_course(course, template):
    """
    Recursively load all courses and return tasks.

    :param course: Path to the course file
    :type course: ``str``
    :param template: Template environment
    :type template: :class:`narrenschiff.templating.Template`
    :return: All tasks from the original, and the included course files
    :rtype: ``list`` of :class:`narrenschiff.task.Task`
    """
    tasks_yaml = _import_current_tasks(course, template)

    tasks = []
    for task in tasks_yaml:
        follow_course = task.get('import_course')
        if follow_course:
            new_tasks = _import_course(follow_course, template)
            for new_task in new_tasks:
                tasks.append(new_task)
        else:
            tasks.append(Task(task))

    return tasks


def _import_current_tasks(course, template):
    """
    Render tasks from the course.

    :param course: Path to the course file
    :type course: ``str``
    :param template: Template environment
    :type template: :class:`narrenschiff.templating.Template`
    :return: Tasks as a list of dictionaries
    :rtype: ``list`` of ``dict``
    """
    tasks_raw = template.render(course)
    tasks_yaml = yaml.safe_load(tasks_raw)
    return tasks_yaml



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click

from narrenschiff.autocomplete import ShellAutocomplete


@click.group()
def autocomplete():
    """
    Manage autocomplete script for your environment.
    \f

    :return: Void
    :rtype: ``None``
    """
    pass


@autocomplete.command()
@click.option('--shell',
              required=False,
              default='bash',
              show_default=True,
              type=str,
              help='Type of the shell.')
def add(shell):
    """
    Add autocomplete script to your environment.
    \f

    :param shell: Type of the shell you are using
    :type shell: ``str``
    :return: Void
    :rtype: ``None``
    """
    autocomplete = ShellAutocomplete()
    autocomplete.add()



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from narrenschiff.modules.mixins import KubectlDryRunMixin
from narrenschiff.modules.common import NarrenschiffModule
from narrenschiff.modules.common import NarrenschiffModuleException
from narrenschiff.templating import Template
from narrenschiff.log import NarrenschiffLogger


logger = NarrenschiffLogger()


class Kustomization(KubectlDryRunMixin, NarrenschiffModule):
    """``kustomization`` module. Wrapper around ``kubectl apply -k dir/``."""

    @property
    def cmd(self):
        logger.info('Executing kustomization task')
        if not isinstance(self.command, str):
            exception = 'This module does not support additional arguments'
            raise NarrenschiffModuleException(exception)

        path = os.path.join(Template().tmp, self.command)
        logger.debug(f'Executing kustomization module on {path}')

        return f'kubectl apply -k "{path}"'



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from setuptools import setup
from setuptools import find_packages

import narrenschiff


with open(os.path.join(os.path.dirname(__file__), 'README.md'), 'r') as readme:
    README = readme.read()


setup(
    name='narrenschiff',
    version=narrenschiff.__version__,
    packages=find_packages('.', exclude=['tests*', 'docs*']),
    description='Configuration management tool for Kubernetes',
    long_description=README,
    long_description_content_type='text/markdown',
    license_files=('LICENSE',),
    license='Apache License, Version 2.0',
    url='https://narrenschiff.xyz',
    project_urls={
        'Documentation': 'https://docs.narrenschiff.xyz',
        'Source': 'https://github.com/narrenorg/narrenschiff',
    },
    maintainer='Petar Nikolovski',
    maintainer_email='petar.nikolovski@protonmail.com',
    classifiers=[
        'Environment :: Console',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX',
        'Topic :: Utilities'
    ],
    install_requires=[
        'click', 'PyYAML', 'Jinja2', 'cryptography', 'colorlog'
    ],
    entry_points="""
        [console_scripts]
        narrenschiff=narrenschiff.narrenschiff:narrenschiff
    """,
)



# Copyright 2021 The Narrenschiff Authors

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import time
import subprocess

import click

from narrenschiff.modules.common import NarrenschiffModule
from narrenschiff.modules.common import NarrenschiffModuleException


class WaitForPod(NarrenschiffModule):
    """Use this module when you need to wait for a pod to become ready."""

    def execute(self):
        timeout = 300  # 5min
        pod_name = self.command['grep_pod_name']
        start_time = time.time()

        while True:
            process = subprocess.run(
                self.cmd,
                shell=True,
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )

            output = process.stdout.decode('utf-8')

            # output, rc = self.subprocess(cmd)
            # if rc: -> sys.exit (something went wrong) print(output)

            r = re.search(r'^{}.*\s+(\d)/\d.*'.format(pod_name), output, re.M)
            if int(r.group(1)) == self.command['threshold_replicas']:
                click.secho('Pod ready', fg='green')
                # self.echo('Pod ready', rc)
                break

            if time.time() - start_time >= timeout:
                raise NarrenschiffModuleException('Timeout exceeded')

            # click.secho('Waiting...', fg='green')
            time.sleep(1)

    @property
    def cmd(self):
        namespace = self.command['namespace']
        return 'kubectl get pods --namespace {}'.format(namespace)

    def dry_run_supported(self, cmd):
        return False
